@@ -127,3 +127,42 @@ test_marshal_STRING__INT (GClosure     *closure,
   g_value_take_string (return_value, v_return);
 }
 
+/* VARIANT:POINTER (./testmarshal.list:4) */
+void
+test_marshal_VARIANT__POINTER (GClosure     *closure,
+                               GValue       *return_value G_GNUC_UNUSED,
+                               guint         n_param_values,
+                               const GValue *param_values,
+                               gpointer      invocation_hint G_GNUC_UNUSED,
+                               gpointer      marshal_data)
+{
+  typedef GVariant* (*GMarshalFunc_VARIANT__POINTER) (gpointer     data1,
+                                                      gpointer     arg_1,
+                                                      gpointer     data2);
+  register GMarshalFunc_VARIANT__POINTER callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  GVariant* v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VARIANT__POINTER) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_pointer (param_values + 1),
+                       data2);
+
+  g_value_take_variant (return_value, v_return);
+}
+
